/**
* Grindstone JavaScript Library v3.3.3
* https://github.com/dzervoudakes/GrindstoneJS
*
* Copyright (c) 2014, 2019 dzervoudakes and contributors
* @license MIT
* https://github.com/dzervoudakes/GrindstoneJS/blob/master/LICENSE
*/

"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

(function (root, lib) {
  if (typeof exports !== 'undefined') {
    return module.exports = lib();
  } else {
    root.Grindstone = root.$ = lib();
  }
})(window, function () {
  var Grindstone = function Grindstone(selector, context) {
    var set = this;

    if (selector) {
      var ctx;

      if (typeof selector === 'string') {
        if (context) {
          if (typeof context === 'string') {
            ctx = document.querySelectorAll(context);
          } else if (utils.isElementArray(context)) {
            ctx = context;
          }

          if (ctx instanceof Array) {
            ctx.forEach(function (item) {
              var foundContexts = document.querySelectorAll(item);
              Array.prototype.forEach.call(foundContexts, function (item) {
                utils.applyContext(item, selector, set);
              });
            });
          } else {
            Array.prototype.forEach.call(ctx, function (item) {
              utils.applyContext(item, selector, set);
            });
          }
        } else {
          set.push.apply(set, document.querySelectorAll(selector));
        }
      } else if (utils.isElementArray(selector)) {
        set.push.apply(set, selector);
      } else {
        set.push(selector);
      }
    }

    this.set = set;
    return this;
  };

  Grindstone.prototype = [];

  var $ = function $(selector, context) {
    return new Grindstone(selector, context);
  };

  $.fn = Grindstone.prototype;

  var utils = function () {
    var applyContext = function applyContext(item, selector, set) {
      var elems = item.querySelectorAll(selector);
      Array.prototype.forEach.call(elems, function (el) {
        if (set.indexOf(el) === -1) {
          set.push(el);
        }
      });
    };

    var children = function children(set, nodeType, selector) {
      var newSet = $();

      for (var i = 0; i < set.length; i++) {
        for (var child = set[i].firstChild; child; child = child.nextSibling) {
          if (nodeType === undefined || nodeType === child.nodeType) {
            if (!selector || $(child).is(selector)) newSet.push(child);
          }
        }
      }

      return newSet;
    };

    var createInteraction = function createInteraction(touchEvt, mouseEvt) {
      return 'ontouchend' in document ? touchEvt : mouseEvt;
    };

    var elementProp = function elementProp(set, propName, selector) {
      return $.fn.map.call(set, function () {
        var find = this;

        while (true) {
          var element = find[propName];
          if (!element) break;

          if (element.nodeType !== 1) {
            find = element;
            continue;
          }

          if (!selector || $(element).is(selector)) {
            return element;
          }

          break;
        }
      });
    };

    var isElementArray = function isElementArray(obj) {
      return obj instanceof Array;
    };

    var matchesFuncName = Element.prototype.matches ? 'matches' : Element.prototype.matchesSelector ? 'matchesSelector' : Element.prototype.webkitMatchesSelector ? 'webkitMatchesSelector' : Element.prototype.mozMatchesSelector ? 'mozMatchesSelector' : Element.prototype.msMatchesSelector ? 'msMatchesSelector' : undefined;
    return {
      applyContext: applyContext,
      children: children,
      createInteraction: createInteraction,
      elementProp: elementProp,
      isElementArray: isElementArray,
      matchesFuncName: matchesFuncName
    };
  }();

  var ajax = function ajax(opts) {
    if (_typeof(opts) !== 'object') {
      throw new Error('XHR properties are not properly defined.');
    }

    var _opts$method = opts.method,
        method = _opts$method === void 0 ? 'GET' : _opts$method,
        _opts$url = opts.url,
        url = _opts$url === void 0 ? '' : _opts$url,
        _opts$async = opts.async,
        async = _opts$async === void 0 ? true : _opts$async,
        _opts$dataType = opts.dataType,
        dataType = _opts$dataType === void 0 ? 'json' : _opts$dataType,
        _opts$body = opts.body,
        body = _opts$body === void 0 ? null : _opts$body,
        _opts$headers = opts.headers,
        headers = _opts$headers === void 0 ? {} : _opts$headers;
    return new Promise(function (resolve, reject) {
      var xmlhttp = new XMLHttpRequest();
      xmlhttp.open(method, url, async);
      xmlhttp.responseType = dataType;
      Object.keys(headers).forEach(function (key) {
        xmlhttp.setRequestHeader(key, headers[key]);
      });

      xmlhttp.onload = function () {
        return resolve(xmlhttp.response);
      };

      xmlhttp.onerror = function () {
        return reject(xmlhttp.response);
      };

      xmlhttp.send(body);
      return xmlhttp;
    });
  };

  $.fn.ajax = ajax;

  var addClass = function addClass(cls) {
    var classes = cls.split(' ');
    this.each(function () {
      var _this = this;

      classes.forEach(function (clsName) {
        _this.classList.add(clsName);
      });
    });
    return this;
  };

  $.fn.addClass = addClass;

  var attr = function attr(attribute, value) {
    var elemAttribute;
    this.each(function () {
      if (value || value === '') {
        this.setAttribute(attribute, value);
      } else {
        elemAttribute = this.getAttribute(attribute);
      }
    });
    return value ? this : elemAttribute;
  };

  $.fn.attr = attr;

  var data = function data(valueName, newValue) {
    if (newValue) {
      this.each(function () {
        $(this).attr("data-".concat(valueName), newValue);
      });
      return this;
    } else {
      var elemValue;
      this.each(function () {
        elemValue = $(this).attr("data-".concat(valueName));
      });
      return elemValue;
    }
  };

  $.fn.data = data;

  var hasAttr = function hasAttr(attribute) {
    var exists = false;
    this.each(function () {
      if (attribute) exists = $(this).attr(attribute) !== null;
    });
    return exists;
  };

  $.fn.hasAttr = hasAttr;

  var hasClass = function hasClass(cls) {
    var hasClass = true;
    var classes = cls.split(' ');
    this.each(function () {
      var _this2 = this;

      classes.forEach(function (clsName) {
        if (!_this2.classList.contains(clsName)) {
          hasClass = false;
          return false;
        }
      });
    });
    return hasClass;
  };

  $.fn.hasClass = hasClass;

  var removeAttr = function removeAttr(attribute) {
    this.each(function () {
      if (attribute) this.removeAttribute(attribute);
    });
    return this;
  };

  $.fn.removeAttr = removeAttr;

  var removeClass = function removeClass(cls) {
    var classes = cls.split(' ');
    this.each(function () {
      var _this3 = this;

      classes.forEach(function (clsName) {
        _this3.classList.remove(clsName);
      });
    });
    return this;
  };

  $.fn.removeClass = removeClass;

  var removeData = function removeData(valueName) {
    this.each(function () {
      $(this).removeAttr("data-".concat(valueName));
    });
    return this;
  };

  $.fn.removeData = removeData;

  var toggleClass = function toggleClass(cls) {
    var classes = cls.split(' ');
    this.each(function () {
      var _this4 = this;

      classes.forEach(function (clsName) {
        _this4.classList.toggle(clsName);
      });
    });
    return this;
  };

  $.fn.toggleClass = toggleClass;

  var each = function each(callback) {
    var set = this.set;
    set = Array.prototype.slice.call(set);
    set.forEach(function (item, index, array) {
      callback.call(item, item, index, array);
    });
    return this;
  };

  $.fn.each = each;

  var eq = function eq(index) {
    return $(this.set[index]);
  };

  $.fn.eq = eq;

  var first = function first() {
    return $(this.set[0]);
  };

  $.fn.first = first;

  var get = function get(index) {
    return this.set[index];
  };

  $.fn.get = get;

  var last = function last() {
    return $(this.set[this.set.length - 1]);
  };

  $.fn.last = last;

  var map = function map(callback) {
    var newSet = $();
    var set = this.set;
    set.forEach(function (item, index, array) {
      var ret = callback.call(item, item, index, array);
      if (ret !== undefined && ret !== null) newSet.push(ret);
    });
    return newSet;
  };

  $.fn.map = map;

  var click = function click(callback) {
    this.each(function () {
      if (callback) {
        this.addEventListener('click', callback, false);
      } else {
        var clickEvent = new Event('click');
        this.dispatchEvent(clickEvent);
      }
    });
    return this;
  };

  $.fn.click = click;

  var doubleTap = function doubleTap(callback) {
    var active, interaction;
    this.each(function () {
      active = false;
      interaction = utils.createInteraction('touchend', 'click');
      $(this).on(interaction, function () {
        if (active) {
          callback();
          return active = false;
        }

        active = true;
        setTimeout(function () {
          return active = false;
        }, 350);
      });
    });
    return this;
  };

  $.fn.doubleTap = doubleTap;

  var focus = function focus(callback) {
    if (typeof callback === 'function') {
      this.each(function () {
        $(this).on('focus', callback);
      });
    } else {
      this.set[0].focus();
    }

    return this;
  };

  $.fn.focus = focus;

  var load = function load(callback) {
    if (typeof callback === 'function') {
      this.each(function () {
        $(this).on('load', callback);
      });
    } else {
      throw new Error('No callback specified for "load()"');
    }

    return this;
  };

  $.fn.load = load;

  var off = function off(action, callback) {
    this.each(function () {
      var _this5 = this;

      var events = action.split(' ');
      events.forEach(function (evt) {
        _this5.removeEventListener(evt, callback, false);
      });
    });
    return this;
  };

  $.fn.off = off;

  var on = function on(action, callback) {
    this.each(function () {
      var _this6 = this;

      var events = action.split(' ');
      events.forEach(function (evt) {
        _this6.addEventListener(evt, callback, false);
      });
    });
    return this;
  };

  $.fn.on = on;

  var ready = function ready(callback) {
    if (typeof callback === 'function') {
      this.each(function () {
        $(this).on('DOMContentLoaded', callback);
      });
    } else {
      throw new Error('No callback specified for "ready()"');
    }

    return this;
  };

  $.fn.ready = ready;

  var resize = function resize(callback) {
    if (typeof callback === 'function') {
      this.each(function () {
        $(this).on('resize', callback);
      });
    } else {
      throw new Error('No callback specified for "resize()"');
    }

    return this;
  };

  $.fn.resize = resize;

  var scroll = function scroll(callback) {
    if (typeof callback === 'function') {
      this.each(function () {
        $(this).on('scroll', callback);
      });
    } else {
      throw new Error('No callback specified for "scroll()"');
    }

    return this;
  };

  $.fn.scroll = scroll;

  var scrollLeft = function scrollLeft(left) {
    var leftOffset;
    this.each(function () {
      switch (this) {
        case window:
          if (typeof left === 'number') {
            this.scrollTo(left, 0);
          } else {
            leftOffset = this.pageXOffset;
          }

          break;

        case document:
          if (typeof left === 'number') {
            this.body.scrollLeft = left;
          } else {
            leftOffset = this.body.scrollLeft;
          }

          break;

        default:
          if (typeof left === 'number') {
            this.scrollLeft = left;
          } else {
            leftOffset = this.scrollLeft;
          }

      }
    });
    return typeof left === 'number' ? this : leftOffset;
  };

  $.fn.scrollLeft = scrollLeft;

  var scrollTop = function scrollTop(top) {
    var topOffset;
    this.each(function () {
      switch (this) {
        case window:
          if (typeof top === 'number') {
            this.scrollTo(0, top);
          } else {
            topOffset = this.pageYOffset;
          }

          break;

        case document:
          if (typeof top === 'number') {
            this.body.scrollTop = top;
          } else {
            topOffset = this.body.scrollTop;
          }

          break;

        default:
          if (typeof top === 'number') {
            this.scrollTop = top;
          } else {
            topOffset = this.scrollTop;
          }

      }
    });
    return typeof top === 'number' ? this : topOffset;
  };

  $.fn.scrollTop = scrollTop;

  var trigger = function trigger(evt) {
    var customEvent = new Event(evt);
    this.each(function () {
      this.dispatchEvent(customEvent);
    });
    return this;
  };

  $.fn.trigger = trigger;

  var filter = function filter(filterBy) {
    return $.fn.map.call(this, function () {
      if ($(this).is(filterBy)) return this;
    });
  };

  $.fn.filter = filter;

  var is = function is(filterBy) {
    var isFunction = typeof filterBy === 'function' ? true : false;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = this[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var item = _step.value;
        var condition = isFunction ? filterBy.call(item, this.indexOf(item), item) : item[utils.matchesFuncName](filterBy);
        if (condition) return true;
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return false;
  };

  $.fn.is = is;

  var not = function not(filterBy) {
    return $.fn.map.call(this, function () {
      if (!$(this).is(filterBy)) return this;
    });
  };

  $.fn.not = not;

  var submit = function submit(callback) {
    if (typeof callback === 'function') {
      this.each(function () {
        $(this).on('submit', callback);
      });
    } else {
      this.each(function () {
        this.submit();
      });
    }

    return this;
  };

  $.fn.submit = submit;

  var val = function val(newValue) {
    if (newValue && typeof newValue === 'string') {
      this.each(function () {
        this.value = newValue;
      });
      return this;
    } else {
      return this.set[0].value;
    }
  };

  $.fn.val = val;

  var after = function after(content) {
    this.each(function () {
      var _this7 = this;

      if (typeof content === 'string') {
        if (content.match(/(<).+(>)/)) {
          this.insertAdjacentHTML('afterend', content);
        } else {
          var dom = document.querySelectorAll(content);
          dom = Array.prototype.slice.call(dom);
          dom.forEach(function (item) {
            _this7.parentNode.insertBefore(item, _this7.nextSibling);
          });
        }
      } else {
        this.parentNode.insertBefore(content, this.nextSibling);
      }
    });
    return this;
  };

  $.fn.after = after;

  var append = function append(content) {
    this.each(function () {
      var _this8 = this;

      if (typeof content === 'string') {
        if (content.match(/(<).+(>)/)) {
          this.innerHTML += content;
        } else {
          var dom = document.querySelectorAll(content);
          dom = Array.prototype.slice.call(dom);
          dom.forEach(function (item) {
            _this8.appendChild(item);
          });
        }
      } else {
        this.appendChild(content);
      }
    });
    return this;
  };

  $.fn.append = append;

  var before = function before(content) {
    this.each(function () {
      var _this9 = this;

      if (typeof content === 'string') {
        if (content.match(/(<).+(>)/)) {
          this.insertAdjacentHTML('beforebegin', content);
        } else {
          var dom = document.querySelectorAll(content);
          dom = Array.prototype.slice.call(dom);
          dom.forEach(function (item) {
            _this9.parentNode.insertBefore(item, _this9);
          });
        }
      } else {
        this.parentNode.insertBefore(content, this);
      }
    });
    return this;
  };

  $.fn.before = before;

  var clone = function clone() {
    return this.map(function () {
      return this.cloneNode(true);
    });
  };

  $.fn.clone = clone;

  var css = function css(styles, value) {
    var returnedStyle, returnStyle;
    this.each(function () {
      if (_typeof(styles) === 'object' && !Array.isArray(styles)) {
        Object.assign(this.style, styles);
      } else if (typeof styles === 'string' && (value === undefined || value === null)) {
        returnedStyle = this.style[styles];
        returnStyle = true;
      } else if (typeof styles === 'string') {
        this.style[styles] = value;
      }
    });
    return returnStyle ? returnedStyle : this;
  };

  $.fn.css = css;

  var empty = function empty() {
    this.each(function () {
      while (this.firstChild) {
        this.removeChild(this.firstChild);
      }
    });
    return this;
  };

  $.fn.empty = empty;

  var height = function height(num) {
    if (typeof num === 'number' || num === 0) {
      this.each(function () {
        this.style.height = "".concat(num, "px");
      });
      return this;
    } else {
      if (this.set[0] === document) {
        return document.body.clientHeight;
      } else if (this.set[0] === window) {
        return window.innerHeight;
      } else {
        return this.set[0].offsetHeight;
      }
    }
  };

  $.fn.height = height;

  var hide = function hide(delay) {
    var _this10 = this;

    if (delay) {
      setTimeout(function () {
        $.fn.hide.call(_this10);
      }, delay);
    } else {
      this.each(function () {
        if (this.style.display !== 'none') {
          if (this.style.display) $(this).data('_prevdisplay', this.style.display);
          this.style.display = 'none';
        }
      });
    }

    return this;
  };

  $.fn.hide = hide;

  var html = function html(content) {
    var text;
    this.each(function () {
      if (content || content === '') {
        this.innerHTML = content;
      } else {
        text = this.innerHTML;
      }
    });
    return content ? this : text;
  };

  $.fn.html = html;

  var prepend = function prepend(content) {
    this.each(function () {
      var _this11 = this;

      if (typeof content === 'string') {
        if (content.match(/(<).+(>)/)) {
          this.insertAdjacentHTML('afterbegin', content);
        } else {
          var dom = document.querySelectorAll(content);
          dom = Array.prototype.slice.call(dom);
          dom.forEach(function (item) {
            _this11.insertBefore(item, _this11.firstChild);
          });
        }
      } else {
        this.insertBefore(content, this.firstChild);
      }
    });
    return this;
  };

  $.fn.prepend = prepend;

  var remove = function remove(target) {
    if (target) {
      var elems = document.querySelectorAll(target);
      elems = Array.prototype.slice.call(elems);
      this.each(function () {
        var _this12 = this;

        elems.forEach(function (el) {
          _this12.removeChild(el);
        });
      });
    } else {
      this.each(function () {
        this.parentNode.removeChild(this);
      });
    }

    return this;
  };

  $.fn.remove = remove;

  var replaceWith = function replaceWith(content) {
    this.each(function () {
      this.outerHTML = content ? content : '';
    });
    return this;
  };

  $.fn.replaceWith = replaceWith;

  var show = function show(delay) {
    var _this13 = this;

    if (delay) {
      setTimeout(function () {
        $.fn.show.call(_this13);
      }, delay);
    } else {
      this.each(function () {
        if (this.style.display === 'none') {
          this.style.display = $(this).data('_prevdisplay') || '';
          $(this).removeData('_prevdisplay');
        }
      });
    }

    return this;
  };

  $.fn.show = show;

  var width = function width(num) {
    if (typeof num === 'number' || num === 0) {
      this.each(function () {
        this.style.width = "".concat(num, "px");
      });
      return this;
    } else {
      if (this.set[0] === document) {
        return document.body.clientWidth;
      } else if (this.set[0] === window) {
        return window.innerWidth;
      } else {
        return this.set[0].offsetWidth;
      }
    }
  };

  $.fn.width = width;

  var wrap = function wrap(structure) {
    this.each(function () {
      if (typeof structure !== 'string' || !structure.startsWith('<') || !structure.endsWith('>')) {
        throw new Error('wrap() string must start with "<" and ending with ">".');
      }

      var contents = this.outerHTML;
      this.outerHTML = structure + contents;
    });
    return this;
  };

  $.fn.wrap = wrap;

  var wrapInner = function wrapInner(structure) {
    this.each(function () {
      if (typeof structure !== 'string' || !structure.startsWith('<') || !structure.endsWith('>')) {
        throw new Error('wrapInner() string must start with "<" and ending with ">".');
      }

      var contents = $(this).html();
      $(this).html(structure + contents);
    });
    return this;
  };

  $.fn.wrapInner = wrapInner;

  var debounce = function debounce(fn, wait, immediate) {
    var timeout;

    var debounce = function debounce() {
      var context = this;
      var args = arguments;

      var later = function later() {
        timeout = null;
        if (!immediate) fn.apply(context, args);
      };

      var callNow = immediate && !timeout;
      clearTimeout(timeout);
      timeout = setTimeout(later, wait);
      if (callNow) fn.apply(context, args);
    };

    return debounce;
  };

  $.debounce = debounce;

  var extend = function extend(target) {
    if (_typeof(target) !== 'object' || !(arguments.length <= 1 ? 0 : arguments.length - 1) === 0) {
      throw new Error('Cannot merge properties into the target: argument is not an object.');
    }

    for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i++) {
      if (_typeof(i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]) !== 'object' || Array.isArray(i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1])) {
        throw new Error('Cannot merge properties into the target: argument is not an object.');
      }

      target = Object.assign(target, i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]);
    }

    return target;
  };

  $.extend = extend;

  var mouseable = function mouseable() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
      hoverClass: 'over',
      activeClass: 'down'
    },
        _ref$hoverClass = _ref.hoverClass,
        hoverClass = _ref$hoverClass === void 0 ? 'over' : _ref$hoverClass,
        _ref$activeClass = _ref.activeClass,
        activeClass = _ref$activeClass === void 0 ? 'down' : _ref$activeClass;

    var events = {
      hover: utils.createInteraction('touchstart', 'mouseenter'),
      remove: utils.createInteraction('touchend', 'mouseleave'),
      down: utils.createInteraction('touchstart', 'mousedown'),
      up: utils.createInteraction('touchend', 'mouseup mouseleave')
    };
    this.each(function () {
      var _this14 = this;

      $(this).on(events.hover, function () {
        $(_this14).addClass(hoverClass);
      }).on(events.remove, function () {
        $(_this14).removeClass(hoverClass);
      }).on(events.down, function () {
        $(_this14).addClass(activeClass);
      }).on(events.up, function () {
        $(_this14).removeClass(activeClass);
      });
    });
    return this;
  };

  $.fn.mouseable = mouseable;

  var offset = function offset(position) {
    if (position !== 'left' && position !== 'top') {
      throw new Error('offset() position must be either "left" or "top".');
    }

    var el = this.set[0];

    if (position === 'left') {
      var offsetLeft = 0;

      do {
        if (!isNaN(el.offsetLeft)) offsetLeft += el.offsetLeft;
        el = el.offsetParent;
      } while (el);

      return offsetLeft;
    } else if (position === 'top') {
      var offsetTop = 0;

      do {
        if (!isNaN(el.offsetTop)) offsetTop += el.offsetTop;
        el = el.offsetParent;
      } while (el);

      return offsetTop;
    }
  };

  $.fn.offset = offset;

  var children = function children(selector) {
    return utils.children(this, 1, selector);
  };

  $.fn.children = children;

  var contents = function contents() {
    return utils.children(this);
  };

  $.fn.contents = contents;

  var next = function next(selector) {
    return utils.elementProp(this, 'nextSibling', selector);
  };

  $.fn.next = next;

  var parent = function parent(selector) {
    return utils.elementProp(this, 'parentNode', selector);
  };

  $.fn.parent = parent;

  var prev = function prev(selector) {
    return utils.elementProp(this, 'previousSibling', selector);
  };

  $.fn.prev = prev;
  return $;
});